import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {HeaderFooterCmsRepository} from "./header-footer-cms.repository";
import {UpdateHeaderFooterCmsDto} from "./dto/update-header-footer-cms.dto";
import {HeaderFooterCmsEntity} from "./entities/header-footer-cms.entity";

@Injectable()
export class HeaderFooterCmsService {
    constructor(
        @InjectRepository(HeaderFooterCmsRepository)
        private readonly repository: HeaderFooterCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateHeaderFooterCmsDto,
        files: {
            siteLogo: UploadedMulterFileI,
        }
    ): Promise<HeaderFooterCmsEntity | Error> {
        if (files && files.siteLogo) {
            const siteLogo: any = await this.doSpaceService.uploadFile(files.siteLogo[0], "HeaderFooterCms/");
            dto.siteLogo = siteLogo;
        }

        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<HeaderFooterCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}