import {EntityRepository, Repository} from "typeorm";
import {HomePageHeroSectionCmsEntity} from "./entities/home-page-hero-section-cms.entity";
import {UpdateHomePageHeroSectionCmsDto} from "./dto/update-home-page-hero-section-cms.dto";

@EntityRepository(HomePageHeroSectionCmsEntity)
export class HomePageHeroSectionCmsRepository extends Repository<HomePageHeroSectionCmsEntity> {

    async createEntity(dto: UpdateHomePageHeroSectionCmsDto): Promise<HomePageHeroSectionCmsEntity | Error> {
        try {
            const {
                title,
                subTitle,
                description,
                buttonOneText,
                buttonOneLink,
                buttonTwoText,
                buttonTwoLink,
                overviewImage
            } = dto;
            const entity = this.create({
                title,
                subTitle,
                description,
                buttonOneText,
                buttonOneLink,
                buttonTwoText,
                buttonTwoLink,
                overviewImage
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(dto: UpdateHomePageHeroSectionCmsDto, id: string): Promise<HomePageHeroSectionCmsEntity | Error> {
        try {
            await this.update(id, {
                title: dto.title,
                subTitle: dto.subTitle,
                description: dto.description,
                buttonOneText: dto.buttonOneText,
                buttonOneLink: dto.buttonOneLink,
                buttonTwoText: dto.buttonTwoText,
                buttonTwoLink: dto.buttonTwoLink,
                overviewImage: dto.overviewImage
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<HomePageHeroSectionCmsEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}