import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateCareerModelDto} from "./dto/create-career-model.dto";
import {UpdateCareerModelDto} from "./dto/update-career-model.dto";
import {CareerModelEntity} from "./entity/career-model.entity";

@EntityRepository(CareerModelEntity)
export class CareerModelRepository extends Repository<CareerModelEntity> {

    async createEntity(dto: CreateCareerModelDto): Promise<CareerModelEntity | Error> {
        try {
            const {
                locationType,
                team,
                title,
                address,
                description,
                salary
            } = dto;
            const entity = this.create({
                locationType,
                team,
                title,
                address,
                description,
                salary
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<CareerModelEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<CareerModelEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateCareerModelDto): Promise<CareerModelEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(CareerModelEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async findCareerByFilterQuery(
        query: any
    ): Promise<any | Error> {
        try {
            const careers = await this.find({
                where: {
                    ...query,
                    isDeleted: false
                }
            });
            return {
                careers
            };
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}