import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateDesignationDto} from "./dto/create-designation.dto";
import {UpdateDesignationDto} from "./dto/update-designation.dto";
import {DesignationEntity} from "./entity/designation.entity";

@EntityRepository(DesignationEntity)
export class DesignationRepository extends Repository<DesignationEntity> {

    async createEntity(dto: CreateDesignationDto): Promise<DesignationEntity | Error> {
        try {
            const {
                name
            } = dto;
            const entity = this.create({
                name
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<DesignationEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<DesignationEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateDesignationDto): Promise<DesignationEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(DesignationEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}