import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {HomePageBrandSectionRepository} from './home-page-brand-section.repository';
import {CreateHomePageBrandSectionDto} from './dto/create-home-page-brand-section.dto';
import {UpdateHomePageBrandSectionDto} from './dto/update-home-page-brand-section.dto';
import {HomePageBrandSectionEntity} from './entity/home-page-brand-section.entity';
import {DoSpacesService} from "../../SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";

@Injectable()
export class HomePageBrandSectionService {
    constructor(
        @InjectRepository(HomePageBrandSectionRepository)
        private readonly repository: HomePageBrandSectionRepository,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    async create(dto: CreateHomePageBrandSectionDto, files: {
        logo?: UploadedMulterFileI,
    }): Promise<HomePageBrandSectionEntity | Error> {
        if (files && files.logo) {
            const url = await this.doSpacesService.uploadFile(files.logo[0], "HomePageBrandSection/");
            dto.logo = url.toString();
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
    }

    async findAll(): Promise<HomePageBrandSectionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async update(id: string, dto: UpdateHomePageBrandSectionDto, files: {
        logo?: UploadedMulterFileI,
    }): Promise<HomePageBrandSectionEntity | Error> {
        if (files && files.logo) {
            const url = await this.doSpacesService.uploadFile(files.logo[0], "HomePageBrandSection/");
            dto.logo = url.toString();
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", data);
    }

    async findOne(id: string): Promise<HomePageBrandSectionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
    }
}
