import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreatePaymentPackageDto} from "./dto/create-payment-package.dto";
import {UpdatePaymentPackageDto} from "./dto/update-payment-package.dto";
import {PaymentPackageEntity} from "./entity/payment-package.entity";

@EntityRepository(PaymentPackageEntity)
export class PaymentPackageRepository extends Repository<PaymentPackageEntity> {

    async createEntity(dto: CreatePaymentPackageDto): Promise<PaymentPackageEntity | Error> {
        try {
            const {
                price,
                packageType,
                packageName,
                description,
                features,
                buttonText,
                buttonLink
            } = dto;
            const entity = this.create({
                price,
                packageType,
                packageName,
                description,
                features,
                buttonText,
                buttonLink
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<PaymentPackageEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<PaymentPackageEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdatePaymentPackageDto): Promise<PaymentPackageEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(PaymentPackageEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}