import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreatePressCoverageContentDto} from "./dto/create-press-coverage-content.dto";
import {UpdatePressCoverageContentDto} from "./dto/update-press-coverage-content.dto";
import {PressCoverageContentEntity} from "./entity/press-coverage-content.entity";

@EntityRepository(PressCoverageContentEntity)
export class PressCoverageContentRepository extends Repository<PressCoverageContentEntity> {
    async createEntity(dto: CreatePressCoverageContentDto): Promise<PressCoverageContentEntity | Error> {
        try {
            const {
                title,
                description,
                image
            } = dto;
            const entity = this.create({
                title,
                description,
                image
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<PressCoverageContentEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdatePressCoverageContentDto): Promise<PressCoverageContentEntity | Error> {
        try {
            await this.update(id, {
                title: dto.title,
                description: dto.description,
                image: dto.image
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<PressCoverageContentEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(PressCoverageContentEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}