import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {FileUploadsService} from 'src/modules/file-uploads/file-uploads.service';
import {SocialLinkRepository} from './social-link.repository';
import {CreateSocialLinkDto} from './dto/create-social-link.dto';
import {SocialLinkEntity} from './entity/social-link.entity';
import {UpdateSocialLinkDto} from './dto/update-social-link.dto';
import {DoSpacesService} from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import {UploadedMulterFileI} from 'src/modules/SpacesModule/SpacesService';

@Injectable()
export class SocialLinkService {
    constructor(
        @InjectRepository(SocialLinkRepository)
        private readonly repository: SocialLinkRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateSocialLinkDto,
        files: {
            icon?: UploadedMulterFileI,
        }
    ): Promise<SocialLinkEntity | Error> {
        if (files && files.icon) {
            const url = await this.doSpacesService.uploadFile(files.icon[0], "SocialLink/");
            dto.icon = url.toString();
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateSocialLinkDto,
        files: {
            icon?: UploadedMulterFileI,
        }
    ): Promise<SocialLinkEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.icon) {
            const url = await this.doSpacesService.uploadFile(files.icon[0], "SocialLink/");
            dto.icon = url.toString();
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<SocialLinkEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<SocialLinkEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
