import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateSubscriptionModelDto} from "./dto/create-subscription-model.dto";
import {UpdateSubscriptionModelDto} from "./dto/update-subscription-model.dto";
import {SubscriptionModelEntity} from "./entity/subscription-model.entity";

@EntityRepository(SubscriptionModelEntity)
export class SubscriptionModelRepository extends Repository<SubscriptionModelEntity> {

    async createEntity(dto: CreateSubscriptionModelDto): Promise<SubscriptionModelEntity | Error> {
        try {
            const {
                email
            } = dto;
            const entity = this.create({
                email
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<SubscriptionModelEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<SubscriptionModelEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateSubscriptionModelDto): Promise<SubscriptionModelEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(SubscriptionModelEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}