import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {CreateEmailNotifierDto} from "./dto/create-email-notifier.dto";
import {UpdateEmailNotifierDto} from "./dto/update-email-notifier.dto";
import {EmailNotifierRepository} from "./email-notifier-repository";
import {EmailNotifiyee} from "./entity/email-notifier.entity";

@Injectable()
export class EmailNotifierService {

    constructor(
        @InjectRepository(EmailNotifierRepository)
        private readonly repository: EmailNotifierRepository
    ) {
    }

    async create(dto: CreateEmailNotifierDto): Promise<EmailNotifiyee | Error> {

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Admin Email notifyee data created", "data", data);
    }

    async findAll(): Promise<EmailNotifiyee[] | Error> {

        const data = await this.repository.findByFilterQuery({});

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Admin Email notifye data fetched", "data", data);
    }


    async update(id: string, dto: UpdateEmailNotifierDto): Promise<EmailNotifiyee | Error> {

        const email: any = await this.repository.findOneEntity(id);
        if (!email) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const data = await this.repository.updateEmail(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Admin Email notifyee data updated", "data", data);

    }


    async findOne(id: string): Promise<EmailNotifiyee[] | Error> {

        const data = await this.repository.findByFilterQuery({id: id});

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Admin Email notifyee data fetched", "data", data);
    }


    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.buildDeletedData(
            data
        );
    }

}
