import { EntityRepository, getConnection, Repository } from "typeorm";
import { EventScheduleEntity } from "./entities/event-schedule.entity";
import { CreateEventScheduleDto } from "./dto/createEventSchedule.dto";

@EntityRepository(EventScheduleEntity)
export class EventScheduleRepository extends Repository<EventScheduleEntity> {
    async createEntity(dto: CreateEventScheduleDto): Promise<EventScheduleEntity | Error> {
        try {
            const entity = this.create(dto);
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<EventScheduleEntity[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<EventScheduleEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findScheduleOfUser(id: string): Promise<EventScheduleEntity[]> {
        return this.createQueryBuilder('event-schedule')
          .where('event-schedule.invitedUserId = :id', { id })
          .orWhere('event-schedule.invitedBy = :id', { id })
          .andWhere('event-schedule.isDeleted = 0')
          .orderBy('event-schedule.startTime', 'ASC')
          .getMany();
      }

    async updateEntity(id: string, dto: any): Promise<EventScheduleEntity | Error> {
        try {
            await this.update(id, { ...dto });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(EventScheduleEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}