import { EntityRepository, getConnection, getManager, Repository } from "typeorm";
import { InvitesEntity } from "./entities/invites.entity";
import { CreateInviteDto } from "./dto/createInvite.dto";

@EntityRepository(InvitesEntity)
export class InvitesRepository extends Repository<InvitesEntity> {
    async createEntity(dto: CreateInviteDto): Promise<InvitesEntity | Error> {
        try {
            const entity = this.create(dto);
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<InvitesEntity[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<InvitesEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findInviteDetails(id: string) {
        try {
            const query =
            `SELECT i.id, u.firstName, u.lastName, t.streetAddress, i.status, i.transactionId
            FROM invites i
            INNER JOIN transaction t ON t.id = i.transactionId
            INNER JOIN users u on u.id = i.invitedBy
            WHERE i.id = '${id}' limit 1`;

            const data = await this.manager.query(query);
            return data?.[0];
        } catch (error) {
            return error;
        }
    }

    async updateEntity(id: string, dto: any): Promise<InvitesEntity | Error> {
        try {
            await this.update(id, { ...dto });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(InvitesEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async getInvitesWithTrxNUser(id: string) {
        let sql = `SELECT i.*, u.firstName, u.lastName, u.image, t.streetAddress FROM invites i 
        LEFT JOIN users u ON i.invitedBy = u.id
        LEFT JOIN transaction t ON i.transactionId = t.id
        WHERE i.invitedUserId = '${id}' 
        and i.status = 'PENDING'
        and i.isDeleted = 0;`;

        const entityManager = getManager();
        const data = await entityManager.query(sql);

        return data;
    }
}