import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {FileUploadsService} from 'src/modules/file-uploads/file-uploads.service';
import {TransactionNotesRepository} from './transaction-notes.repository';
import {CreateTransactionNotesDto} from './dto/create-transaction-notes.dto';
import {TransactionNotesEntity} from './entity/transaction-notes.entity';
import {UpdateTransactionNotesDto} from './dto/update-transaction-notes.dto';
import {DoSpacesService} from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import {UploadedMulterFileI} from 'src/modules/SpacesModule/SpacesService';

@Injectable()
export class TransactionNotesService {
    constructor(
        @InjectRepository(TransactionNotesRepository)
        private readonly repository: TransactionNotesRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateTransactionNotesDto,
        files: {
            file?: UploadedMulterFileI,
        },
        request: any
    ): Promise<TransactionNotesEntity | Error> {
        if (files && files.file) {
            const url = await this.doSpacesService.uploadFile(files.file[0], "TransactionNotes/");
            dto.file = url.toString();
        }
        dto.userId = request.user.id;
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateTransactionNotesDto,
        files: {
            file?: UploadedMulterFileI,
        },
        request: any
    ): Promise<TransactionNotesEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.file) {
            const url = await this.doSpacesService.uploadFile(files.file[0], "TransactionNotes/");
            dto.file = url.toString();
        }
        dto.userId = request.user.id;
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(transactionId: any): Promise<TransactionNotesEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({transactionId: transactionId});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<TransactionNotesEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
