import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { TransactionRepository } from '../transaction/transaction.repository';
import { CreateTransactionTimelineDetailsDto } from './dto/create-transaction-timeline-details.dto';
import { TransactionTimelineDetailsEntity } from './entity/transaction-timeline-details.entity';
import { TransactionTimelineDetailsRepository } from './transaction-timeline-details.repository';
import { UpdateTransactionTimelineDetailsDto } from './dto/update-transaction-timeline-details.dto';

@Injectable()
export class TransactionTimelineDetailsService {
    constructor(
        @InjectRepository(TransactionTimelineDetailsRepository)
        private readonly repository: TransactionTimelineDetailsRepository,
        @InjectRepository(TransactionRepository)
        private readonly transactionRepository: TransactionRepository,
    ) {
    }

    async create(
        dto: CreateTransactionTimelineDetailsDto,
        request: any
    ): Promise<TransactionTimelineDetailsEntity | Error> {
        const transaction: any = await this.transactionRepository.findByFilterQuery({ id: dto.transactionId });
        if (!transaction.length) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateTransactionTimelineDetailsDto,
        request: any
    ): Promise<TransactionTimelineDetailsEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const { transactionId, ...updateDto } = dto;
        const data = await this.repository.updateEntity(id, updateDto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<TransactionTimelineDetailsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<TransactionTimelineDetailsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
