import { EntityRepository, getConnection, getManager, In, Repository } from "typeorm";
import {CreateTransactionDto} from "./dto/create-transaction.dto";
import {UpdateTransactionDto} from "./dto/update-transaction.dto";
import {TransactionEntity} from "./entity/transaction.entity";

@EntityRepository(TransactionEntity)
export class TransactionRepository extends Repository<TransactionEntity> {

    async createEntity(dto: CreateTransactionDto): Promise<TransactionEntity | Error> {
        try {
            const {
                streetAddress,
                city,
                state,
                postalCode,
                clientName,
                price,
                createDate,
                closingDate,
                additionalNotes,
                userId,
                userName,
                status
            } = dto;
            const entity = this.create({
                streetAddress,
                city,
                state,
                postalCode,
                clientName,
                price,
                createDate,
                closingDate,
                additionalNotes,
                userId,
                userName,
                status
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<TransactionEntity[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async siteTransactionQuery(transactionIds : string[]): Promise<any[] | Error> {
        try {
            return await this.find({
                where: { id: In(transactionIds), isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async transactionReportQuery(transactionIds : string[]): Promise<any[] | Error> {
        try {
            return await this.find({
                where: { id: In(transactionIds), isDeleted: false },
                select: ["streetAddress", "clientName", "price", "createDate", "closingDate", "status"]
            });
        } catch (e) {
            return e;
        }
    }

    // async findByFilterQuery(query: any, offset: number, limit: number): Promise<TransactionEntity[] | Error> {
    //     try {
    //         return await this.find({
    //             where: {...query, isDeleted: false},
    //             skip: offset,
    //             take: limit,
    //         });
    //     } catch (e) {
    //         return e;
    //     }
    // }

    async findOneByFilterQuery(query: any): Promise<TransactionEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<TransactionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateTransactionDto): Promise<TransactionEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(TransactionEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async filterTransaction(userId: string, paginationDto: any): Promise<any | Error> {
        try {
            const {page, limit, sort, sortName, searchName, startDate, endDate} = paginationDto;
            let whereQuery: any = `t.id in (SELECT transactionId FROM party WHERE userId='${userId}')`;
            let orderByQuery: any = ``;

            if (sortName != "Default" && sortName != "") {
                orderByQuery = `ORDER BY ${sortName} ${sort}`
            }

            if (searchName) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and t.state like '%${searchName}%' OR 
                    t.city LIKE '%${searchName}%' OR
                    t.clientName LIKE '%${searchName}%' OR 
                    t.streetAddress LIKE '%${searchName}%' OR 
                    t.postalCode LIKE '%${searchName}%' `;
                } else {
                    whereQuery += ` t.state like '%${searchName}%' OR 
                    t.city LIKE '%${searchName}%' OR 
                    t.clientName LIKE '%${searchName}%' OR
                    t.streetAddress LIKE '%${searchName}%' OR
                    t.postalCode LIKE '%${searchName}%' `;
                }
            }

            if (startDate && endDate) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and t.createdAt between '${startDate}' and '${endDate}'`;
                } else {
                    whereQuery += ` t.createdAt between '${startDate}' and '${endDate}'`;
                }
            }

            if (whereQuery.length > 3) {
                whereQuery += ` and t.isDeleted=0 `;
            } else {
                whereQuery += ` t.isDeleted=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            } else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;

            if (page > 1) {
                offset = (page - 1) * limit;
            }

            let sql = `SELECT t.* FROM transaction as t ${conditionQuery} ${orderByQuery} limit ${offset}, ${limit};`;
            let sqlTotal = `SELECT count(*) total FROM transaction as t ${conditionQuery}`;

            // console.log(sql);
            // console.log(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData?.[0]?.total;

            if (count > 0) {
                const pageCount = Math.ceil(count / limit);

                return {
                    page: page,
                    limit: limit,
                    pageCount: pageCount,
                    totalDataFound: count,
                    data,
                };
            } else {
                return {};
            }
        } catch (e) {
            console.log(e);
            return e;
        }
    }
}